@@ -1,7 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
-  Atomic KMSDRM backend by Manuel Alfayate Corchete <redwindwanderer@gmail.com>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -31,93 +30,93 @@
 #include <unistd.h>
 #include <xf86drm.h>
 #include <xf86drmMode.h>
-
 #include <gbm.h>
-#include <assert.h>
-#if SDL_VIDEO_OPENGL_EGL
 #include <EGL/egl.h>
-#include <EGL/eglext.h>
+
+#ifndef DRM_MODE_FB_MODIFIERS
+#define DRM_MODE_FB_MODIFIERS	2
 #endif
 
-/****************************************************************************************/
-/* Driverdata pointers are void struct* used to store backend-specific variables        */
-/* and info that supports the SDL-side structs like SDL Display Devices, SDL_Windows... */
-/* which need to be "supported" with backend-side info and mechanisms to work.          */ 
-/****************************************************************************************/
+#ifndef DRM_MODE_PAGE_FLIP_ASYNC
+#define DRM_MODE_PAGE_FLIP_ASYNC    2
+#endif
+
+#ifndef DRM_MODE_OBJECT_CONNECTOR
+#define DRM_MODE_OBJECT_CONNECTOR   0xc0c0c0c0
+#endif
+
+#ifndef DRM_MODE_OBJECT_CRTC
+#define DRM_MODE_OBJECT_CRTC        0xcccccccc
+#endif
+
+#ifndef DRM_CAP_ASYNC_PAGE_FLIP
+#define DRM_CAP_ASYNC_PAGE_FLIP 7
+#endif
+
+#ifndef DRM_CAP_CURSOR_WIDTH
+#define DRM_CAP_CURSOR_WIDTH    8
+#endif
+
+#ifndef DRM_CAP_CURSOR_HEIGHT
+#define DRM_CAP_CURSOR_HEIGHT   9
+#endif
+
+#ifndef GBM_FORMAT_ARGB8888
+#define GBM_FORMAT_ARGB8888  ((uint32_t)('A') | ((uint32_t)('R') << 8) | ((uint32_t)('2') << 16) | ((uint32_t)('4') << 24))
+#define GBM_BO_USE_CURSOR   (1 << 1)
+#define GBM_BO_USE_WRITE    (1 << 3)
+#define GBM_BO_USE_LINEAR   (1 << 4)
+#endif
 
 typedef struct SDL_VideoData
 {
-    int devindex;               /* device index that was passed on creation */
-    int drm_fd;                 /* DRM file desc */
-    char devpath[32];           /* DRM dev path. */
+    int devindex;     /* device index that was passed on creation */
+    int drm_fd;       /* DRM file desc */
+    char devpath[32]; /* DRM dev path. */
 
     struct gbm_device *gbm_dev;
 
-    SDL_Window **windows;
-    unsigned int max_windows;
-    unsigned int num_windows;
+    SDL_bool video_init;             /* Has VideoInit succeeded? */
+    SDL_bool vulkan_mode;            /* Are we in Vulkan mode? One VK window is enough to be. */
+    SDL_bool async_pageflip_support; /* Does the hardware support async. pageflips? */
 
-    SDL_bool video_init;        /* Has VideoInit succeeded? */
+    SDL_Window **windows;
+    int max_windows;
+    int num_windows;
+
+    /* Even if we have several displays, we only have to
+       open 1 FD and create 1 gbm device. */
+    SDL_bool gbm_init;
 
 } SDL_VideoData;
 
-typedef struct plane {
-    drmModePlane *plane;
-    drmModeObjectProperties *props;
-    drmModePropertyRes **props_info;
-} plane;
+typedef struct SDL_DisplayModeData
+{
+    int mode_index;
+} SDL_DisplayModeData;
 
-typedef struct crtc {
-    drmModeCrtc *crtc;
-    drmModeObjectProperties *props;
-    drmModePropertyRes **props_info;
-} crtc;
-
-typedef struct connector {
-    drmModeConnector *connector;
-    drmModeObjectProperties *props;
-    drmModePropertyRes **props_info;
-} connector;
-
-/* More general driverdata info that gives support and substance to the SDL_Display. */
 typedef struct SDL_DisplayData
 {
+    drmModeConnector *connector;
+    drmModeCrtc *crtc;
     drmModeModeInfo mode;
-    drmModeModeInfo preferred_mode;
-    uint32_t atomic_flags;
+    drmModeModeInfo original_mode;
+    drmModeModeInfo fullscreen_mode;
 
-    plane *display_plane;
-    plane *cursor_plane;
-    crtc *crtc;
-    connector *connector;
-
-    /* Central atomic request list, used for the prop
-       changeset related to pageflip in SwapWindow. */ 
-    drmModeAtomicReq *atomic_req;
-
-    int kms_in_fence_fd;
-    int kms_out_fence_fd;
-
-    EGLSyncKHR kms_fence;
-    EGLSyncKHR gpu_fence;
-
-#if SDL_VIDEO_OPENGL_EGL
-    EGLSurface old_egl_surface;
-#endif
-
-    SDL_bool modeset_pending;
-    SDL_bool gbm_init;
+    drmModeCrtc *saved_crtc; /* CRTC to restore on quit */
+    SDL_bool saved_vrr;
 
     /* DRM & GBM cursor stuff lives here, not in an SDL_Cursor's driverdata struct,
        because setting/unsetting up these is done on window creation/destruction,
        where we may not have an SDL_Cursor at all (so no SDL_Cursor driverdata).
        There's only one cursor GBM BO because we only support one cursor. */
     struct gbm_bo *cursor_bo;
+    int cursor_bo_drm_fd;
     uint64_t cursor_w, cursor_h;
 
+    SDL_bool default_cursor_init;
 } SDL_DisplayData;
 
-/* Driverdata info that gives KMSDRM-side support and substance to the SDL_Window. */
 typedef struct SDL_WindowData
 {
     SDL_VideoData *viddata;
@@ -129,65 +128,24 @@ typedef struct SDL_WindowData
     struct gbm_bo *bo;
     struct gbm_bo *next_bo;
 
-#if SDL_VIDEO_OPENGL_EGL
+    SDL_bool waiting_for_flip;
+    SDL_bool double_buffer;
+
     EGLSurface egl_surface;
-#endif
-
-    /* For scaling and AR correction. */
-    int32_t src_w;
-    int32_t src_h;
-    int32_t output_w;
-    int32_t output_h;
-    int32_t output_x;
-
-    /* This dictates what approach we'll use for SwapBuffers. */
-    int (*swap_window)(_THIS, SDL_Window * window);
-
+    SDL_bool egl_surface_dirty;
 } SDL_WindowData;
 
-typedef struct SDL_DisplayModeData
-{
-    int mode_index;
-} SDL_DisplayModeData;
-
 typedef struct KMSDRM_FBInfo
 {
-    int drm_fd;         /* DRM file desc */
-    uint32_t fb_id;     /* DRM framebuffer ID */
+    int drm_fd;     /* DRM file desc */
+    uint32_t fb_id; /* DRM framebuffer ID */
 } KMSDRM_FBInfo;
 
-typedef struct KMSDRM_PlaneInfo
-{
-    struct plane *plane;
-    uint32_t fb_id;
-    uint32_t crtc_id;
-    int32_t src_x;
-    int32_t src_y;
-    int32_t src_w;
-    int32_t src_h;
-    int32_t crtc_x;
-    int32_t crtc_y;
-    int32_t crtc_w;
-    int32_t crtc_h;
-} KMSDRM_PlaneInfo;
-
 /* Helper functions */
-int KMSDRM_CreateEGLSurface(_THIS, SDL_Window * window);
+int KMSDRM_CreateSurfaces(_THIS, SDL_Window *window);
 KMSDRM_FBInfo *KMSDRM_FBFromBO(_THIS, struct gbm_bo *bo);
-
-/* Atomic functions that are used from SDL_kmsdrmopengles.c and SDL_kmsdrmmouse.c */
-void drm_atomic_set_plane_props(struct KMSDRM_PlaneInfo *info); 
-
-void drm_atomic_waitpending(_THIS);
-int drm_atomic_commit(_THIS, SDL_bool blocking);
-int add_plane_property(drmModeAtomicReq *req, struct plane *plane,
-                             const char *name, uint64_t value);
-int add_crtc_property(drmModeAtomicReq *req, struct crtc *crtc,
-                             const char *name, uint64_t value);
-int add_connector_property(drmModeAtomicReq *req, struct connector *connector,
-                             const char *name, uint64_t value);
-int setup_plane(_THIS, struct plane **plane, uint32_t plane_type);
-void free_plane(struct plane **plane);
+KMSDRM_FBInfo *KMSDRM_FBFromBO2(_THIS, struct gbm_bo *bo, int w, int h);
+SDL_bool KMSDRM_WaitPageflip(_THIS, SDL_WindowData *windata);
 
 /****************************************************************************/
 /* SDL_VideoDevice functions declaration                                    */
@@ -205,13 +163,14 @@ void KMSDRM_SetWindowIcon(_THIS, SDL_Window * window, SDL_Surface * icon);
 void KMSDRM_SetWindowPosition(_THIS, SDL_Window * window);
 void KMSDRM_SetWindowSize(_THIS, SDL_Window * window);
 void KMSDRM_SetWindowFullscreen(_THIS, SDL_Window * window, SDL_VideoDisplay * _display, SDL_bool fullscreen);
+int KMSDRM_SetWindowGammaRamp(_THIS, SDL_Window * window, const Uint16 * ramp);
+int KMSDRM_GetWindowGammaRamp(_THIS, SDL_Window * window, Uint16 * ramp);
 void KMSDRM_ShowWindow(_THIS, SDL_Window * window);
 void KMSDRM_HideWindow(_THIS, SDL_Window * window);
 void KMSDRM_RaiseWindow(_THIS, SDL_Window * window);
 void KMSDRM_MaximizeWindow(_THIS, SDL_Window * window);
 void KMSDRM_MinimizeWindow(_THIS, SDL_Window * window);
 void KMSDRM_RestoreWindow(_THIS, SDL_Window * window);
-void KMSDRM_SetWindowGrab(_THIS, SDL_Window * window, SDL_bool grabbed);
 void KMSDRM_DestroyWindow(_THIS, SDL_Window * window);
 
 /* Window manager function */
@@ -222,11 +181,11 @@ SDL_bool KMSDRM_GetWindowWMInfo(_THIS, SDL_Window * window,
 int KMSDRM_GLES_LoadLibrary(_THIS, const char *path);
 void *KMSDRM_GLES_GetProcAddress(_THIS, const char *proc);
 void KMSDRM_GLES_UnloadLibrary(_THIS);
-SDL_GLContext KMSDRM_GLES_CreateContext(_THIS, SDL_Window * window);
-int KMSDRM_GLES_MakeCurrent(_THIS, SDL_Window * window, SDL_GLContext context);
+SDL_GLContext KMSDRM_GLES_CreateContext(_THIS, SDL_Window *window);
+int KMSDRM_GLES_MakeCurrent(_THIS, SDL_Window *window, SDL_GLContext context);
 int KMSDRM_GLES_SetSwapInterval(_THIS, int interval);
 int KMSDRM_GLES_GetSwapInterval(_THIS);
-int KMSDRM_GLES_SwapWindow(_THIS, SDL_Window * window);
+int KMSDRM_GLES_SwapWindow(_THIS, SDL_Window *window);
 void KMSDRM_GLES_DeleteContext(_THIS, SDL_GLContext context);
 
 #endif /* __SDL_KMSDRMVIDEO_H__ */
