@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -32,17 +32,16 @@ ThreadFunc(void *data)
 {
     /* Set the child thread error string */
     SDL_SetError("Thread %s (%lu) had a problem: %s",
-                 (char *) data, SDL_ThreadID(), "nevermind");
+                 (char *)data, SDL_ThreadID(), "nevermind");
     while (alive) {
-        SDL_Log("Thread '%s' is alive!\n", (char *) data);
+        SDL_Log("Thread '%s' is alive!\n", (char *)data);
         SDL_Delay(1 * 1000);
     }
     SDL_Log("Child thread error string: %s\n", SDL_GetError());
-    return (0);
+    return 0;
 }
 
-int
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     SDL_Thread *thread;
 
@@ -52,15 +51,21 @@ main(int argc, char *argv[])
     /* Load the SDL library */
     if (SDL_Init(0) < 0) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't initialize SDL: %s\n", SDL_GetError());
-        return (1);
+        return 1;
     }
 
     /* Set the error value for the main thread */
     SDL_SetError("No worries");
 
+    if (SDL_getenv("SDL_TESTS_QUICK") != NULL) {
+        SDL_Log("Not running slower tests");
+        SDL_Quit();
+        return 0;
+    }
+
     alive = 1;
     thread = SDL_CreateThread(ThreadFunc, NULL, "#1");
-    if (thread == NULL) {
+    if (!thread) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't create thread: %s\n", SDL_GetError());
         quit(1);
     }
@@ -72,5 +77,5 @@ main(int argc, char *argv[])
     SDL_Log("Main thread error string: %s\n", SDL_GetError());
 
     SDL_Quit();
-    return (0);
+    return 0;
 }
